@@ -1,216 +1,146 @@
 <?php
-$drivers["sqlite"] = "SQLite 3";
-$drivers["sqlite2"] = "SQLite 2";
+namespace Adminer;
 
-if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
-	$possible_drivers = array((isset($_GET["sqlite"]) ? "SQLite3" : "SQLite"), "PDO_SQLite");
-	define("DRIVER", (isset($_GET["sqlite"]) ? "sqlite" : "sqlite2"));
-	if (class_exists(isset($_GET["sqlite"]) ? "SQLite3" : "SQLiteDatabase")) {
-		if (isset($_GET["sqlite"])) {
+add_driver("sqlite", "SQLite");
 
-			class Min_SQLite {
-				var $extension = "SQLite3", $server_info, $affected_rows, $errno, $error, $_link;
+if (isset($_GET["sqlite"])) {
+	define('Adminer\DRIVER', "sqlite");
 
-				function __construct($filename) {
-					$this->_link = new SQLite3($filename);
-					$version = $this->_link->version();
-					$this->server_info = $version["versionString"];
-				}
+	if (class_exists("SQLite3") && $_GET["ext"] != "pdo") {
+		abstract class SqliteDb extends SqlDb {
+			public $extension = "SQLite3";
+			private $link;
 
-				function query($query) {
-					$result = @$this->_link->query($query);
-					$this->error = "";
-					if (!$result) {
-						$this->errno = $this->_link->lastErrorCode();
-						$this->error = $this->_link->lastErrorMsg();
-						return false;
-					} elseif ($result->numColumns()) {
-						return new Min_Result($result);
-					}
-					$this->affected_rows = $this->_link->changes();
-					return true;
-				}
-
-				function quote($string) {
-					return (is_utf8($string)
-						? "'" . $this->_link->escapeString($string) . "'"
-						: "x'" . reset(unpack('H*', $string)) . "'"
-					);
-				}
-
-				function store_result() {
-					return $this->_result;
-				}
-
-				function result($query, $field = 0) {
-					$result = $this->query($query);
-					if (!is_object($result)) {
-						return false;
-					}
-					$row = $result->_result->fetchArray();
-					return $row[$field];
-				}
+			function attach(?string $filename, string $username, string $password): string {
+				$this->link = new \SQLite3($filename);
+				$version = $this->link->version();
+				$this->server_info = $version["versionString"];
+				return '';
 			}
 
-			class Min_Result {
-				var $_result, $_offset = 0, $num_rows;
-
-				function __construct($result) {
-					$this->_result = $result;
-				}
-
-				function fetch_assoc() {
-					return $this->_result->fetchArray(SQLITE3_ASSOC);
-				}
-
-				function fetch_row() {
-					return $this->_result->fetchArray(SQLITE3_NUM);
-				}
-
-				function fetch_field() {
-					$column = $this->_offset++;
-					$type = $this->_result->columnType($column);
-					return (object) array(
-						"name" => $this->_result->columnName($column),
-						"type" => $type,
-						"charsetnr" => ($type == SQLITE3_BLOB ? 63 : 0), // 63 - binary
-					);
-				}
-
-				function __desctruct() {
-					return $this->_result->finalize();
+			function query(string $query, bool $unbuffered = false) {
+				$result = @$this->link->query($query);
+				$this->error = "";
+				if (!$result) {
+					$this->errno = $this->link->lastErrorCode();
+					$this->error = $this->link->lastErrorMsg();
+					return false;
+				} elseif ($result->numColumns()) {
+					return new Result($result);
 				}
+				$this->affected_rows = $this->link->changes();
+				return true;
 			}
 
-		} else {
+			function quote(string $string): string {
+				return (is_utf8($string)
+					? "'" . $this->link->escapeString($string) . "'"
+					: "x'" . first(unpack('H*', $string)) . "'"
+				);
+			}
+		}
 
-			class Min_SQLite {
-				var $extension = "SQLite", $server_info, $affected_rows, $error, $_link;
+		class Result {
+			public $num_rows;
+			private $result, $offset = 0;
 
-				function __construct($filename) {
-					$this->server_info = sqlite_libversion();
-					$this->_link = new SQLiteDatabase($filename);
-				}
-
-				function query($query, $unbuffered = false) {
-					$method = ($unbuffered ? "unbufferedQuery" : "query");
-					$result = @$this->_link->$method($query, SQLITE_BOTH, $error);
-					$this->error = "";
-					if (!$result) {
-						$this->error = $error;
-						return false;
-					} elseif ($result === true) {
-						$this->affected_rows = $this->changes();
-						return true;
-					}
-					return new Min_Result($result);
-				}
-
-				function quote($string) {
-					return "'" . sqlite_escape_string($string) . "'";
-				}
-
-				function store_result() {
-					return $this->_result;
-				}
-
-				function result($query, $field = 0) {
-					$result = $this->query($query);
-					if (!is_object($result)) {
-						return false;
-					}
-					$row = $result->_result->fetch();
-					return $row[$field];
-				}
+			function __construct($result) {
+				$this->result = $result;
 			}
 
-			class Min_Result {
-				var $_result, $_offset = 0, $num_rows;
-
-				function __construct($result) {
-					$this->_result = $result;
-					if (method_exists($result, 'numRows')) { // not available in unbuffered query
-						$this->num_rows = $result->numRows();
-					}
-				}
-
-				function fetch_assoc() {
-					$row = $this->_result->fetch(SQLITE_ASSOC);
-					if (!$row) {
-						return false;
-					}
-					$return = array();
-					foreach ($row as $key => $val) {
-						$return[($key[0] == '"' ? idf_unescape($key) : $key)] = $val;
-					}
-					return $return;
-				}
-
-				function fetch_row() {
-					return $this->_result->fetch(SQLITE_NUM);
-				}
-
-				function fetch_field() {
-					$name = $this->_result->fieldName($this->_offset++);
-					$pattern = '(\[.*]|"(?:[^"]|"")*"|(.+))';
-					if (preg_match("~^($pattern\\.)?$pattern\$~", $name, $match)) {
-						$table = ($match[3] != "" ? $match[3] : idf_unescape($match[2]));
-						$name = ($match[5] != "" ? $match[5] : idf_unescape($match[4]));
-					}
-					return (object) array(
-						"name" => $name,
-						"orgname" => $name,
-						"orgtable" => $table,
-					);
-				}
-
+			function fetch_assoc() {
+				return $this->result->fetchArray(SQLITE3_ASSOC);
 			}
 
+			function fetch_row() {
+				return $this->result->fetchArray(SQLITE3_NUM);
+			}
+
+			function fetch_field(): \stdClass {
+				$column = $this->offset++;
+				$type = $this->result->columnType($column);
+				return (object) array(
+					"name" => $this->result->columnName($column),
+					"type" => ($type == SQLITE3_TEXT ? 15 : 0),
+					"charsetnr" => ($type == SQLITE3_BLOB ? 63 : 0), // 63 - binary
+				);
+			}
+
+			function __destruct() {
+				$this->result->finalize();
+			}
 		}
 
 	} elseif (extension_loaded("pdo_sqlite")) {
-		class Min_SQLite extends Min_PDO {
-			var $extension = "PDO_SQLite";
+		abstract class SqliteDb extends PdoDb {
+			public $extension = "PDO_SQLite";
 
-			function __construct($filename) {
+			function attach(?string $filename, string $username, string $password): string {
 				$this->dsn(DRIVER . ":$filename", "", "");
+				$this->query("PRAGMA foreign_keys = 1");
+				$this->query("PRAGMA busy_timeout = 500");
+				return '';
 			}
 		}
 
 	}
 
-	if (class_exists("Min_SQLite")) {
-		class Min_DB extends Min_SQLite {
-
-			function __construct() {
-				parent::__construct(":memory:");
+	if (class_exists('Adminer\SqliteDb')) {
+		class Db extends SqliteDb {
+			function attach(?string $filename, string $username, string $password): string {
+				parent::attach($filename, $username, $password);
 				$this->query("PRAGMA foreign_keys = 1");
+				$this->query("PRAGMA busy_timeout = 500");
+				return '';
 			}
 
-			function select_db($filename) {
-				if (is_readable($filename) && $this->query("ATTACH " . $this->quote(preg_match("~(^[/\\\\]|:)~", $filename) ? $filename : dirname($_SERVER["SCRIPT_FILENAME"]) . "/$filename") . " AS a")) { // is_readable - SQLite 3
-					parent::__construct($filename);
-					$this->query("PRAGMA foreign_keys = 1");
-					return true;
+			function select_db(string $filename): bool {
+				if (is_readable($filename) && $this->query("ATTACH " . $this->quote(preg_match("~(^[/\\\\]|:)~", $filename) ? $filename : dirname($_SERVER["SCRIPT_FILENAME"]) . "/$filename") . " AS a")) {
+					return !self::attach($filename, '', '');
 				}
 				return false;
 			}
-
-			function multi_query($query) {
-				return $this->_result = $this->query($query);
-			}
-
-			function next_result() {
-				return false;
-			}
 		}
 	}
 
 
 
-	class Min_Driver extends Min_SQL {
+	class Driver extends SqlDriver {
+		static $extensions = array("SQLite3", "PDO_SQLite");
+		static $jush = "sqlite";
 
-		function insertUpdate($table, $rows, $primary) {
+		protected $types = array(array("integer" => 0, "real" => 0, "numeric" => 0, "text" => 0, "blob" => 0));
+
+		public $insertFunctions = array(); // "text" => "date('now')/time('now')/datetime('now')",
+		public $editFunctions = array(
+			"integer|real|numeric" => "+/-",
+			// "text" => "date/time/datetime",
+			"text" => "||",
+		);
+
+		public $operators = array("=", "<", ">", "<=", ">=", "!=", "LIKE", "LIKE %%", "IN", "IS NULL", "NOT LIKE", "NOT IN", "IS NOT NULL", "SQL"); // REGEXP can be user defined function
+		public $functions = array("hex", "length", "lower", "round", "unixepoch", "upper");
+		public $grouping = array("avg", "count", "count distinct", "group_concat", "max", "min", "sum");
+
+		static function connect(?string $server, string $username, string $password) {
+			if ($password != "") {
+				return lang('Database does not support password.');
+			}
+			return parent::connect(":memory:", "", "");
+		}
+
+		function __construct(Db $connection) {
+			parent::__construct($connection);
+			if (min_version(3.31, 0, $connection)) {
+				$this->generated = array("STORED", "VIRTUAL");
+			}
+		}
+
+		function structuredTypes(): array {
+			return array_keys($this->types[0]);
+		}
+
+		function insertUpdate(string $table, array $rows, array $primary) {
 			$values = array();
 			foreach ($rows as $set) {
 				$values[] = "(" . implode(", ", $set) . ")";
@@ -218,7 +148,7 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 			return queries("REPLACE INTO " . table($table) . " (" . implode(", ", array_keys(reset($rows))) . ") VALUES\n" . implode(",\n", $values));
 		}
 
-		function tableHelp($name) {
+		function tableHelp(string $name, bool $is_view = false) {
 			if ($name == "sqlite_sequence") {
 				return "fileformat2.html#seqtab";
 			}
@@ -227,6 +157,20 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 			}
 		}
 
+		function checkConstraints(string $table): array {
+			preg_match_all('~ CHECK *(\( *(((?>[^()]*[^() ])|(?1))*) *\))~', get_val("SELECT sql FROM sqlite_master WHERE type = 'table' AND name = " . q($table), 0, $this->conn), $matches); //! could be inside a comment
+			return array_combine($matches[2], $matches[2]);
+		}
+
+		function allFields(): array {
+			$return = array();
+			foreach (tables_list() as $table => $type) {
+				foreach (fields($table) as $field) {
+					$return[$table][] = $field;
+				}
+			}
+			return $return;
+		}
 	}
 
 
@@ -239,38 +183,23 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 		return idf_escape($idf);
 	}
 
-	function connect() {
-		global $adminer;
-		list(, , $password) = $adminer->credentials();
-		if ($password != "") {
-			return lang('Database does not support password.');
-		}
-		return new Min_DB;
-	}
-
-	function get_databases() {
+	function get_databases($flush) {
 		return array();
 	}
 
 	function limit($query, $where, $limit, $offset = 0, $separator = " ") {
-		return " $query$where" . ($limit !== null ? $separator . "LIMIT $limit" . ($offset ? " OFFSET $offset" : "") : "");
+		return " $query$where" . ($limit ? $separator . "LIMIT $limit" . ($offset ? " OFFSET $offset" : "") : "");
 	}
 
 	function limit1($table, $query, $where, $separator = "\n") {
-		global $connection;
-		return (preg_match('~^INTO~', $query) || $connection->result("SELECT sqlite_compileoption_used('ENABLE_UPDATE_DELETE_LIMIT')")
+		return (preg_match('~^INTO~', $query) || get_val("SELECT sqlite_compileoption_used('ENABLE_UPDATE_DELETE_LIMIT')")
 			? limit($query, $where, 1, 0, $separator)
 			: " $query WHERE rowid = (SELECT rowid FROM " . table($table) . $where . $separator . "LIMIT 1)" //! use primary key in tables with WITHOUT rowid
 		);
 	}
 
 	function db_collation($db, $collations) {
-		global $connection;
-		return $connection->result("PRAGMA encoding"); // there is no database list so $db == DB
-	}
-
-	function engines() {
-		return array();
+		return get_val("PRAGMA encoding"); // there is no database list so $db == DB
 	}
 
 	function logged_user() {
@@ -286,16 +215,15 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 	}
 
 	function table_status($name = "") {
-		global $connection;
 		$return = array();
 		foreach (get_rows("SELECT name AS Name, type AS Engine, 'rowid' AS Oid, '' AS Auto_increment FROM sqlite_master WHERE type IN ('table', 'view') " . ($name != "" ? "AND name = " . q($name) : "ORDER BY name")) as $row) {
-			$row["Rows"] = $connection->result("SELECT COUNT(*) FROM " . idf_escape($row["Name"]));
+			$row["Rows"] = get_val("SELECT COUNT(*) FROM " . idf_escape($row["Name"]));
 			$return[$row["Name"]] = $row;
 		}
-		foreach (get_rows("SELECT * FROM sqlite_sequence", null, "") as $row) {
+		foreach (get_rows("SELECT * FROM sqlite_sequence" . ($name != "" ? " WHERE name = " . q($name) : ""), null, "") as $row) {
 			$return[$row["name"]]["Auto_increment"] = $row["seq"];
 		}
-		return ($name != "" ? $return[$name] : $return);
+		return $return;
 	}
 
 	function is_view($table_status) {
@@ -303,15 +231,13 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 	}
 
 	function fk_support($table_status) {
-		global $connection;
-		return !$connection->result("SELECT sqlite_compileoption_used('OMIT_FOREIGN_KEY')");
+		return !get_val("SELECT sqlite_compileoption_used('OMIT_FOREIGN_KEY')");
 	}
 
 	function fields($table) {
-		global $connection;
 		$return = array();
 		$primary = "";
-		foreach (get_rows("PRAGMA table_info(" . table($table) . ")") as $row) {
+		foreach (get_rows("PRAGMA table_" . (min_version(3.31) ? "x" : "") . "info(" . table($table) . ")") as $row) {
 			$name = $row["name"];
 			$type = strtolower($row["type"]);
 			$default = $row["dflt_value"];
@@ -319,9 +245,9 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 				"field" => $name,
 				"type" => (preg_match('~int~i', $type) ? "integer" : (preg_match('~char|clob|text~i', $type) ? "text" : (preg_match('~blob~i', $type) ? "blob" : (preg_match('~real|floa|doub~i', $type) ? "real" : "numeric")))),
 				"full_type" => $type,
-				"default" => (preg_match("~'(.*)'~", $default, $match) ? str_replace("''", "'", $match[1]) : ($default == "NULL" ? null : $default)),
+				"default" => (preg_match("~^'(.*)'$~", $default, $match) ? str_replace("''", "'", $match[1]) : ($default == "NULL" ? null : $default)),
 				"null" => !$row["notnull"],
-				"privileges" => array("select" => 1, "insert" => 1, "update" => 1),
+				"privileges" => array("select" => 1, "insert" => 1, "update" => 1, "where" => 1, "order" => 1),
 				"primary" => $row["pk"],
 			);
 			if ($row["pk"]) {
@@ -333,24 +259,28 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 				$primary = $name;
 			}
 		}
-		$sql = $connection->result("SELECT sql FROM sqlite_master WHERE type = 'table' AND name = " . q($table));
-		preg_match_all('~(("[^"]*+")+|[a-z0-9_]+)\s+text\s+COLLATE\s+(\'[^\']+\'|\S+)~i', $sql, $matches, PREG_SET_ORDER);
+		$sql = get_val("SELECT sql FROM sqlite_master WHERE type = 'table' AND name = " . q($table));
+		$idf = '(("[^"]*+")+|[a-z0-9_]+)';
+		preg_match_all('~' . $idf . '\s+text\s+COLLATE\s+(\'[^\']+\'|\S+)~i', $sql, $matches, PREG_SET_ORDER);
 		foreach ($matches as $match) {
 			$name = str_replace('""', '"', preg_replace('~^"|"$~', '', $match[1]));
 			if ($return[$name]) {
 				$return[$name]["collation"] = trim($match[3], "'");
 			}
 		}
+		preg_match_all('~' . $idf . '\s.*GENERATED ALWAYS AS \((.+)\) (STORED|VIRTUAL)~i', $sql, $matches, PREG_SET_ORDER);
+		foreach ($matches as $match) {
+			$name = str_replace('""', '"', preg_replace('~^"|"$~', '', $match[1]));
+			$return[$name]["default"] = $match[3];
+			$return[$name]["generated"] = strtoupper($match[4]);
+		}
 		return $return;
 	}
 
 	function indexes($table, $connection2 = null) {
-		global $connection;
-		if (!is_object($connection2)) {
-			$connection2 = $connection;
-		}
+		$connection2 = connection($connection2);
 		$return = array();
-		$sql = $connection2->result("SELECT sql FROM sqlite_master WHERE type = 'table' AND name = " . q($table));
+		$sql = get_val("SELECT sql FROM sqlite_master WHERE type = 'table' AND name = " . q($table), 0, $connection2);
 		if (preg_match('~\bPRIMARY\s+KEY\s*\((([^)"]+|"[^"]*"|`[^`]*`)++)~i', $sql, $match)) {
 			$return[""] = array("type" => "PRIMARY", "columns" => array(), "lengths" => array(), "descs" => array());
 			preg_match_all('~((("[^"]*+")+|(?:`[^`]*+`)+)|(\S+))(\s+(ASC|DESC))?(,\s*|$)~i', $match[1], $matches, PREG_SET_ORDER);
@@ -395,7 +325,6 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 		$return = array();
 		foreach (get_rows("PRAGMA foreign_key_list(" . table($table) . ")") as $row) {
 			$foreign_key = &$return[$row["id"]];
-			//! idf_unescape in SQLite2
 			if (!$foreign_key) {
 				$foreign_key = $row;
 			}
@@ -406,8 +335,7 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 	}
 
 	function view($name) {
-		global $connection;
-		return array("select" => preg_replace('~^(?:[^`"[]+|`[^`]*`|"[^"]*")* AS\s+~iU', '', $connection->result("SELECT sql FROM sqlite_master WHERE name = " . q($name)))); //! identifiers may be inside []
+		return array("select" => preg_replace('~^(?:[^`"[]+|`[^`]*`|"[^"]*")* AS\s+~iU', '', get_val("SELECT sql FROM sqlite_master WHERE type = 'view' AND name = " . q($name)))); //! identifiers may be inside []
 	}
 
 	function collations() {
@@ -419,34 +347,32 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 	}
 
 	function error() {
-		global $connection;
-		return h($connection->error);
+		return h(connection()->error);
 	}
 
 	function check_sqlite_name($name) {
 		// avoid creating PHP files on unsecured servers
-		global $connection;
 		$extensions = "db|sdb|sqlite";
 		if (!preg_match("~^[^\\0]*\\.($extensions)\$~", $name)) {
-			$connection->error = lang('Please use one of the extensions %s.', str_replace("|", ", ", $extensions));
+			connection()->error = lang('Please use one of the extensions %s.', str_replace("|", ", ", $extensions));
 			return false;
 		}
 		return true;
 	}
 
 	function create_database($db, $collation) {
-		global $connection;
 		if (file_exists($db)) {
-			$connection->error = lang('File exists.');
+			connection()->error = lang('File exists.');
 			return false;
 		}
 		if (!check_sqlite_name($db)) {
 			return false;
 		}
 		try {
-			$link = new Min_SQLite($db);
-		} catch (Exception $ex) {
-			$connection->error = $ex->getMessage();
+			$link = new Db();
+			$link->attach($db, '', '');
+		} catch (\Exception $ex) {
+			connection()->error = $ex->getMessage();
 			return false;
 		}
 		$link->query('PRAGMA encoding = "UTF-8"');
@@ -456,11 +382,10 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 	}
 
 	function drop_databases($databases) {
-		global $connection;
-		$connection->__construct(":memory:"); // to unlock file, doesn't work in PDO on Windows
+		connection()->attach(":memory:", '', ''); // to unlock file, doesn't work in PDO on Windows
 		foreach ($databases as $db) {
 			if (!@unlink($db)) {
-				$connection->error = lang('File exists.');
+				connection()->error = lang('File exists.');
 				return false;
 			}
 		}
@@ -468,17 +393,16 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 	}
 
 	function rename_database($name, $collation) {
-		global $connection;
 		if (!check_sqlite_name($name)) {
 			return false;
 		}
-		$connection->__construct(":memory:");
-		$connection->error = lang('File exists.');
+		connection()->attach(":memory:", '', '');
+		connection()->error = lang('File exists.');
 		return @rename(DB, $name);
 	}
 
 	function auto_increment() {
-		return " PRIMARY KEY" . (DRIVER == "sqlite" ? " AUTOINCREMENT" : "");
+		return " PRIMARY KEY AUTOINCREMENT";
 	}
 
 	function alter_table($table, $name, $fields, $foreign, $comment, $engine, $collation, $auto_increment, $partitioning) {
@@ -508,16 +432,32 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 			if ($table != $name && !queries("ALTER TABLE " . table($table) . " RENAME TO " . table($name))) {
 				return false;
 			}
-		} elseif (!recreate_table($table, $name, $alter, $originals, $foreign)) {
+		} elseif (!recreate_table($table, $name, $alter, $originals, $foreign, $auto_increment)) {
 			return false;
 		}
 		if ($auto_increment) {
+			queries("BEGIN");
 			queries("UPDATE sqlite_sequence SET seq = $auto_increment WHERE name = " . q($name)); // ignores error
+			if (!connection()->affected_rows) {
+				queries("INSERT INTO sqlite_sequence (name, seq) VALUES (" . q($name) . ", $auto_increment)");
+			}
+			queries("COMMIT");
 		}
 		return true;
 	}
 
-	function recreate_table($table, $name, $fields, $originals, $foreign, $indexes = array()) {
+	/** Recreate table
+	* @param string $table original name
+	* @param string $name new name
+	* @param list<list<string>> $fields [process_field()], empty to preserve
+	* @param string[] $originals [$original => idf_escape($new_column)], empty to preserve
+	* @param string[] $foreign [format_foreign_key()], empty to preserve
+	* @param numeric-string $auto_increment set auto_increment to this value, "" to preserve
+	* @param list<array{string, string, list<string>|'DROP'}> $indexes [[$type, $name, $columns]], empty to preserve
+	* @param string $drop_check CHECK constraint to drop
+	* @param string $add_check CHECK constraint to add
+	*/
+	function recreate_table(string $table, string $name, array $fields, array $originals, array $foreign, string $auto_increment = "", $indexes = array(), string $drop_check = "", string $add_check = ""): bool {
 		if ($table != "") {
 			if (!$fields) {
 				foreach (fields($table) as $key => $field) {
@@ -574,29 +514,46 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 			}
 			queries("BEGIN");
 		}
-		foreach ($fields as $key => $field) {
-			$fields[$key] = "  " . implode($field);
+		$changes = array();
+		foreach ($fields as $field) {
+			if (preg_match('~GENERATED~', $field[3])) {
+				unset($originals[array_search($field[0], $originals)]);
+			}
+			$changes[] = "  " . implode($field);
 		}
-		$fields = array_merge($fields, array_filter($foreign));
-		if (!queries("CREATE TABLE " . table($table != "" ? "adminer_$name" : $name) . " (\n" . implode(",\n", $fields) . "\n)")) {
-			// implicit ROLLBACK to not overwrite $connection->error
+		$changes = array_merge($changes, array_filter($foreign));
+		foreach (driver()->checkConstraints($table) as $check) {
+			if ($check != $drop_check) {
+				$changes[] = "  CHECK ($check)";
+			}
+		}
+		if ($add_check) {
+			$changes[] = "  CHECK ($add_check)";
+		}
+		$temp_name = ($table == $name ? "adminer_$name" : $name);
+		if (!queries("CREATE TABLE " . table($temp_name) . " (\n" . implode(",\n", $changes) . "\n)")) {
+			// implicit ROLLBACK to not overwrite connection()->error
 			return false;
 		}
 		if ($table != "") {
-			if ($originals && !queries("INSERT INTO " . table("adminer_$name") . " (" . implode(", ", $originals) . ") SELECT " . implode(", ", array_map('idf_escape', array_keys($originals))) . " FROM " . table($table))) {
+			if ($originals && !queries("INSERT INTO " . table($temp_name) . " (" . implode(", ", $originals) . ") SELECT " . implode(", ", array_map('Adminer\idf_escape', array_keys($originals))) . " FROM " . table($table))) {
 				return false;
 			}
 			$triggers = array();
 			foreach (triggers($table) as $trigger_name => $timing_event) {
-				$trigger = trigger($trigger_name);
+				$trigger = trigger($trigger_name, $table);
 				$triggers[] = "CREATE TRIGGER " . idf_escape($trigger_name) . " " . implode(" ", $timing_event) . " ON " . table($name) . "\n$trigger[Statement]";
 			}
-			if (!queries("DROP TABLE " . table($table))) { // drop before creating indexes and triggers to allow using old names
+			$auto_increment = $auto_increment ? "" : get_val("SELECT seq FROM sqlite_sequence WHERE name = " . q($table)); // if $auto_increment is set then it will be updated later
+			if (
+				!queries("DROP TABLE " . table($table)) // drop before creating indexes and triggers to allow using old names
+				|| ($table == $name && !queries("ALTER TABLE " . table($temp_name) . " RENAME TO " . table($name)))
+				|| !alter_indexes($name, $indexes)
+			) {
 				return false;
 			}
-			queries("ALTER TABLE " . table("adminer_$name") . " RENAME TO " . table($name));
-			if (!alter_indexes($name, $indexes)) {
-				return false;
+			if ($auto_increment) {
+				queries("UPDATE sqlite_sequence SET seq = $auto_increment WHERE name = " . q($name)); // ignores error
 			}
 			foreach ($triggers as $trigger) {
 				if (!queries($trigger)) {
@@ -619,14 +576,15 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 	function alter_indexes($table, $alter) {
 		foreach ($alter as $primary) {
 			if ($primary[0] == "PRIMARY") {
-				return recreate_table($table, $table, array(), array(), array(), $alter);
+				return recreate_table($table, $table, array(), array(), array(), "", $alter);
 			}
 		}
 		foreach (array_reverse($alter) as $val) {
-			if (!queries($val[2] == "DROP"
+			if (
+				!queries($val[2] == "DROP"
 				? "DROP INDEX " . idf_escape($val[1])
-				: index_sql($table, $val[0], $val[1], "(" . implode(", ", $val[2]) . ")")
-			)) {
+				: index_sql($table, $val[0], $val[1], "(" . implode(", ", $val[2]) . ")"))
+			) {
 				return false;
 			}
 		}
@@ -649,8 +607,7 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 		return false;
 	}
 
-	function trigger($name) {
-		global $connection;
+	function trigger($name, $table) {
 		if ($name == "") {
 			return array("Statement" => "BEGIN\n\t;\nEND");
 		}
@@ -658,14 +615,14 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 		$trigger_options = trigger_options();
 		preg_match(
 			"~^CREATE\\s+TRIGGER\\s*$idf\\s*(" . implode("|", $trigger_options["Timing"]) . ")\\s+([a-z]+)(?:\\s+OF\\s+($idf))?\\s+ON\\s*$idf\\s*(?:FOR\\s+EACH\\s+ROW\\s)?(.*)~is",
-			$connection->result("SELECT sql FROM sqlite_master WHERE type = 'trigger' AND name = " . q($name)),
+			get_val("SELECT sql FROM sqlite_master WHERE type = 'trigger' AND name = " . q($name)),
 			$match
 		);
 		$of = $match[3];
 		return array(
 			"Timing" => strtoupper($match[1]),
 			"Event" => strtoupper($match[2]) . ($of ? " OF" : ""),
-			"Of" => ($of[0] == '`' || $of[0] == '"' ? idf_unescape($of) : $of),
+			"Of" => idf_unescape($of),
 			"Trigger" => $name,
 			"Statement" => $match[4],
 		);
@@ -675,7 +632,7 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 		$return = array();
 		$trigger_options = trigger_options();
 		foreach (get_rows("SELECT * FROM sqlite_master WHERE type = 'trigger' AND tbl_name = " . q($table)) as $row) {
-			preg_match('~^CREATE\s+TRIGGER\s*(?:[^`"\s]+|`[^`]*`|"[^"]*")+\s*(' . implode("|", $trigger_options["Timing"]) . ')\s*(.*)\s+ON\b~iU', $row["sql"], $match);
+			preg_match('~^CREATE\s+TRIGGER\s*(?:[^`"\s]+|`[^`]*`|"[^"]*")+\s*(' . implode("|", $trigger_options["Timing"]) . ')\s*(.*?)\s+ON\b~i', $row["sql"], $match);
 			$return[$row["name"]] = array($match[1], $match[2]);
 		}
 		return $return;
@@ -693,9 +650,8 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 		return queries("BEGIN");
 	}
 
-	function last_id() {
-		global $connection;
-		return $connection->result("SELECT LAST_INSERT_ROWID()");
+	function last_id($result) {
+		return get_val("SELECT LAST_INSERT_ROWID()");
 	}
 
 	function explain($connection, $query) {
@@ -705,30 +661,17 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 	function found_rows($table_status, $where) {
 	}
 
-	function types() {
+	function types(): array {
 		return array();
 	}
 
-	function schemas() {
-		return array();
-	}
-
-	function get_schema() {
-		return "";
-	}
-
-	function set_schema($scheme) {
-		return true;
-	}
-
 	function create_sql($table, $auto_increment, $style) {
-		global $connection;
-		$return = $connection->result("SELECT sql FROM sqlite_master WHERE type IN ('table', 'view') AND name = " . q($table));
+		$return = get_val("SELECT sql FROM sqlite_master WHERE type IN ('table', 'view') AND name = " . q($table));
 		foreach (indexes($table) as $name => $index) {
 			if ($name == '') {
 				continue;
 			}
-			$return .= ";\n\n" . index_sql($table, $index['type'], $name, "(" . implode(", ", array_map('idf_escape', $index['columns'])) . ")");
+			$return .= ";\n\n" . index_sql($table, $index['type'], $name, "(" . implode(", ", array_map('Adminer\idf_escape', $index['columns'])) . ")");
 		}
 		return $return;
 	}
@@ -745,10 +688,15 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 	}
 
 	function show_variables() {
-		global $connection;
 		$return = array();
-		foreach (array("auto_vacuum", "cache_size", "count_changes", "default_cache_size", "empty_result_callbacks", "encoding", "foreign_keys", "full_column_names", "fullfsync", "journal_mode", "journal_size_limit", "legacy_file_format", "locking_mode", "page_size", "max_page_count", "read_uncommitted", "recursive_triggers", "reverse_unordered_selects", "secure_delete", "short_column_names", "synchronous", "temp_store", "temp_store_directory", "schema_version", "integrity_check", "quick_check") as $key) {
-			$return[$key] = $connection->result("PRAGMA $key");
+		foreach (get_rows("PRAGMA pragma_list") as $row) {
+			$name = $row["name"];
+			if ($name != "pragma_list" && $name != "compile_options") {
+				$return[$name] = array($name, '');
+				foreach (get_rows("PRAGMA $name") as $row) {
+					$return[$name][1] .= implode(", ", $row) . "\n";
+				}
+			}
 		}
 		return $return;
 	}
@@ -756,8 +704,7 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 	function show_status() {
 		$return = array();
 		foreach (get_vals("PRAGMA compile_options") as $option) {
-			list($key, $val) = explode("=", $option, 2);
-			$return[$key] = $val;
+			$return[] = explode("=", $option, 2) + array('', '');
 		}
 		return $return;
 	}
@@ -770,23 +717,6 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 	}
 
 	function support($feature) {
-		return preg_match('~^(columns|database|drop_col|dump|indexes|descidx|move_col|sql|status|table|trigger|variables|view|view_trigger)$~', $feature);
+		return preg_match('~^(check|columns|database|drop_col|dump|indexes|descidx|move_col|sql|status|table|trigger|variables|view|view_trigger)$~', $feature);
 	}
-
-	$jush = "sqlite";
-	$types = array("integer" => 0, "real" => 0, "numeric" => 0, "text" => 0, "blob" => 0);
-	$structured_types = array_keys($types);
-	$unsigned = array();
-	$operators = array("=", "<", ">", "<=", ">=", "!=", "LIKE", "LIKE %%", "IN", "IS NULL", "NOT LIKE", "NOT IN", "IS NOT NULL", "SQL"); // REGEXP can be user defined function
-	$functions = array("hex", "length", "lower", "round", "unixepoch", "upper");
-	$grouping = array("avg", "count", "count distinct", "group_concat", "max", "min", "sum");
-	$edit_functions = array(
-		array(
-			// "text" => "date('now')/time('now')/datetime('now')",
-		), array(
-			"integer|real|numeric" => "+/-",
-			// "text" => "date/time/datetime",
-			"text" => "||",
-		)
-	);
 }
