@@ -1,3 +1,4 @@
+'use strict';
 
 /** Get first element by selector
 * @param string
@@ -14,17 +15,17 @@ function qs(selector, context) {
 * @return HTMLElement
 */
 function qsl(selector, context) {
-	var els = qsa(selector, context || document);
+	const els = qsa(selector, context);
 	return els[els.length - 1];
 }
 
 /** Get all elements by selector
 * @param string
-* @param HTMLElement
+* @param [HTMLElement] defaults to document
 * @return NodeList
 */
 function qsa(selector, context) {
-	return context.querySelectorAll(selector);
+	return (context || document).querySelectorAll(selector);
 }
 
 /** Return a function calling fn with the next arguments
@@ -33,7 +34,7 @@ function qsa(selector, context) {
 * @return function with preserved this
 */
 function partial(fn) {
-	var args = Array.apply(null, arguments).slice(1);
+	const args = Array.apply(null, arguments).slice(1);
 	return function () {
 		return fn.apply(this, args);
 	};
@@ -45,7 +46,7 @@ function partial(fn) {
 * @return function with preserved this
 */
 function partialArg(fn) {
-	var args = Array.apply(null, arguments);
+	const args = Array.apply(null, arguments);
 	return function (arg) {
 		args[0] = arg;
 		return fn.apply(this, args);
@@ -57,7 +58,7 @@ function partialArg(fn) {
 * @param Object
 */
 function mixin(target, source) {
-	for (var key in source) {
+	for (const key in source) {
 		target[key] = source[key];
 	}
 }
@@ -65,11 +66,11 @@ function mixin(target, source) {
 /** Add or remove CSS class
 * @param HTMLElement
 * @param string
-* @param [bool]
+* @param [boolean]
 */
 function alterClass(el, className, enable) {
 	if (el) {
-		el.className = el.className.replace(RegExp('(^|\\s)' + className + '(\\s|$)'), '$2') + (enable ? ' ' + className : '');
+		el.classList[enable ? 'add' : 'remove'](className);
 	}
 }
 
@@ -78,18 +79,17 @@ function alterClass(el, className, enable) {
 * @return boolean false
 */
 function toggle(id) {
-	var el = qs('#' + id);
-	el.className = (el.className == 'hidden' ? '' : 'hidden');
+	const el = qs('#' + id);
+	el && el.classList.toggle('hidden');
 	return false;
 }
 
 /** Set permanent cookie
 * @param string
 * @param number
-* @param string optional
 */
 function cookie(assign, days) {
-	var date = new Date();
+	const date = new Date();
 	date.setDate(date.getDate() + days);
 	document.cookie = assign + '; expires=' + date;
 }
@@ -101,26 +101,23 @@ function cookie(assign, days) {
 */
 function verifyVersion(current, url, token) {
 	cookie('adminer_version=0', 1);
-	var iframe = document.createElement('iframe');
+	const iframe = document.createElement('iframe');
 	iframe.src = 'https://www.adminer.org/version/?current=' + current;
 	iframe.frameBorder = 0;
 	iframe.marginHeight = 0;
 	iframe.scrolling = 'no';
 	iframe.style.width = '7ex';
 	iframe.style.height = '1.25em';
-	if (window.postMessage && window.addEventListener) {
-		iframe.style.display = 'none';
-		addEventListener('message', function (event) {
-			if (event.origin == 'https://www.adminer.org') {
-				var match = /version=(.+)/.exec(event.data);
-				if (match) {
-					cookie('adminer_version=' + match[1], 1);
-					ajax(url + 'script=version', function () {
-					}, event.data + '&token=' + token);
-				}
+	iframe.style.display = 'none';
+	addEventListener('message', event => {
+		if (event.origin == 'https://www.adminer.org') {
+			const match = /version=(.+)/.exec(event.data);
+			if (match) {
+				cookie('adminer_version=' + match[1], 1);
+				ajax(url + 'script=version', () => { }, event.data + '&token=' + token);
 			}
-		}, false);
-	}
+		}
+	}, false);
 	qs('#version').appendChild(iframe);
 }
 
@@ -132,17 +129,17 @@ function selectValue(select) {
 	if (!select.selectedIndex) {
 		return select.value;
 	}
-	var selected = select.options[select.selectedIndex];
+	const selected = select.options[select.selectedIndex];
 	return ((selected.attributes.value || {}).specified ? selected.value : selected.text);
 }
 
 /** Verify if element has a specified tag name
 * @param HTMLElement
 * @param string regular expression
-* @return bool
+* @return boolean
 */
 function isTag(el, tag) {
-	var re = new RegExp('^(' + tag + ')$', 'i');
+	const re = new RegExp('^(' + tag + ')$', 'i');
 	return el && re.test(el.tagName);
 }
 
@@ -162,7 +159,7 @@ function parentTag(el, tag) {
 * @param HTMLInputElement
 */
 function trCheck(el) {
-	var tr = parentTag(el, 'tr');
+	const tr = parentTag(el, 'tr');
 	alterClass(tr, 'checked', el.checked);
 	if (el.form && el.form['all'] && el.form['all'].onclick) { // Opera treats form.all as document.all
 		el.form['all'].onclick();
@@ -176,11 +173,9 @@ function trCheck(el) {
 */
 function selectCount(id, count) {
 	setHtml(id, (count === '' ? '' : '(' + (count + '').replace(/\B(?=(\d{3})+$)/g, thousandsSeparator) + ')'));
-	var el = qs('#' + id);
+	const el = qs('#' + id);
 	if (el) {
-		var inputs = qsa('input', el.parentNode.parentNode);
-		for (var i = 0; i < inputs.length; i++) {
-			var input = inputs[i];
+		for (const input of qsa('input', el.parentNode.parentNode)) {
 			if (input.type == 'submit') {
 				input.disabled = (count == '0');
 			}
@@ -193,11 +188,10 @@ function selectCount(id, count) {
 * @this HTMLInputElement
 */
 function formCheck(name) {
-	var elems = this.form.elements;
-	for (var i=0; i < elems.length; i++) {
-		if (name.test(elems[i].name)) {
-			elems[i].checked = this.checked;
-			trCheck(elems[i]);
+	for (const elem of this.form.elements) {
+		if (name.test(elem.name)) {
+			elem.checked = this.checked;
+			trCheck(elem);
 		}
 	}
 }
@@ -205,9 +199,8 @@ function formCheck(name) {
 /** Check all rows in <table class="checkable">
 */
 function tableCheck() {
-	var inputs = qsa('table.checkable td:first-child input', document);
-	for (var i=0; i < inputs.length; i++) {
-		trCheck(inputs[i]);
+	for (const input of qsa('table.checkable td:first-child input')) {
+		trCheck(input);
 	}
 }
 
@@ -215,7 +208,7 @@ function tableCheck() {
 * @param string
 */
 function formUncheck(id) {
-	var el = qs('#' + id);
+	const el = qs('#' + id);
 	el.checked = false;
 	trCheck(el);
 }
@@ -225,11 +218,10 @@ function formUncheck(id) {
 * @param RegExp
 * @return number
 */
-function formChecked(el, name) {
-	var checked = 0;
-	var elems = el.form.elements;
-	for (var i=0; i < elems.length; i++) {
-		if (name.test(elems[i].name) && elems[i].checked) {
+function formChecked(input, name) {
+	let checked = 0;
+	for (const el of input.form.elements) {
+		if (name.test(el.name) && el.checked) {
 			checked++;
 		}
 	}
@@ -241,15 +233,15 @@ function formChecked(el, name) {
 * @param [boolean] force click
 */
 function tableClick(event, click) {
-	var td = parentTag(getTarget(event), 'td');
-	var text;
+	const td = parentTag(event.target, 'td');
+	let text;
 	if (td && (text = td.getAttribute('data-text'))) {
 		if (selectClick.call(td, event, +text, td.getAttribute('data-warning'))) {
 			return;
 		}
 	}
 	click = (click || !window.getSelection || getSelection().isCollapsed);
-	var el = getTarget(event);
+	let el = event.target;
 	while (!isTag(el, 'tr')) {
 		if (isTag(el, 'table|a|input|textarea')) {
 			if (el.type != 'checkbox') {
@@ -278,7 +270,7 @@ function tableClick(event, click) {
 	trCheck(el);
 }
 
-var lastChecked;
+let lastChecked;
 
 /** Shift-click on checkbox for multiple selection.
 * @param MouseEvent
@@ -289,11 +281,9 @@ function checkboxClick(event) {
 		return;
 	}
 	if (event.shiftKey && (!lastChecked || lastChecked.name == this.name)) {
-		var checked = (lastChecked ? lastChecked.checked : true);
-		var inputs = qsa('input', parentTag(this, 'table'));
-		var checking = !lastChecked;
-		for (var i=0; i < inputs.length; i++) {
-			var input = inputs[i];
+		const checked = (lastChecked ? lastChecked.checked : true);
+		let checking = !lastChecked;
+		for (const input of qsa('input', parentTag(this, 'table'))) {
 			if (input.name === this.name) {
 				if (checking) {
 					input.checked = checked;
@@ -317,7 +307,7 @@ function checkboxClick(event) {
 * @param string undefined to set parentNode to empty string
 */
 function setHtml(id, html) {
-	var el = qs('[id="' + id.replace(/[\\"]/g, '\\$&') + '"]'); // database name is used as ID
+	const el = qs('[id="' + id.replace(/[\\"]/g, '\\$&') + '"]'); // database name is used as ID
 	if (el) {
 		if (html == null) {
 			el.parentNode.innerHTML = '';
@@ -332,8 +322,8 @@ function setHtml(id, html) {
 * @return number
 */
 function nodePosition(el) {
-	var pos = 0;
-	while (el = el.previousSibling) {
+	let pos = 0;
+	while ((el = el.previousSibling)) {
 		pos++;
 	}
 	return pos;
@@ -356,7 +346,7 @@ function pageClick(href, page) {
 * @this HTMLElement
 */
 function menuOver(event) {
-	var a = getTarget(event);
+	const a = event.target;
 	if (isTag(a, 'a|span') && a.offsetLeft + a.offsetWidth > a.parentNode.offsetWidth - 15) { // 15 - ellipsis
 		this.style.overflow = 'visible';
 	}
@@ -366,7 +356,7 @@ function menuOver(event) {
 * @this HTMLElement
 */
 function menuOut() {
-	this.style.overflow = 'auto';
+	this.style.overflow = 'hidden';
 }
 
 
@@ -375,23 +365,21 @@ function menuOut() {
 * @this HTMLSelectElement
 */
 function selectAddRow() {
-	var field = this;
-	var row = cloneNode(field.parentNode);
+	const field = this;
+	const row = cloneNode(field.parentNode);
 	field.onchange = selectFieldChange;
 	field.onchange();
-	var selects = qsa('select', row);
-	for (var i=0; i < selects.length; i++) {
-		selects[i].name = selects[i].name.replace(/[a-z]\[\d+/, '$&1');
-		selects[i].selectedIndex = 0;
+	for (const select of qsa('select', row)) {
+		select.name = select.name.replace(/[a-z]\[\d+/, '$&1');
+		select.selectedIndex = 0;
 	}
-	var inputs = qsa('input', row);
-	for (var i=0; i < inputs.length; i++) {
-		inputs[i].name = inputs[i].name.replace(/[a-z]\[\d+/, '$&1');
-		inputs[i].className = '';
-		if (inputs[i].type == 'checkbox') {
-			inputs[i].checked = false;
+	for (const input of qsa('input', row)) {
+		input.name = input.name.replace(/[a-z]\[\d+/, '$&1');
+		input.className = '';
+		if (input.type == 'checkbox') {
+			input.checked = false;
 		} else {
-			inputs[i].value = '';
+			input.value = '';
 		}
 	}
 	field.parentNode.parentNode.appendChild(row);
@@ -403,8 +391,7 @@ function selectAddRow() {
 */
 function selectSearchKeydown(event) {
 	if (event.keyCode == 13 || event.keyCode == 10) {
-		this.onsearch = function () {
-		};
+		this.onsearch = () => { };
 	}
 }
 
@@ -419,15 +406,14 @@ function selectSearchSearch() {
 
 
 
-/** Toggles column context menu
+/** Toggle column context menu
 * @param [string] extra class name
 * @this HTMLElement
 */
 function columnMouse(className) {
-	var spans = qsa('span', this);
-	for (var i=0; i < spans.length; i++) {
-		if (/column/.test(spans[i].className)) {
-			spans[i].className = 'column' + (className || '');
+	for (const span of qsa('span', this)) {
+		if (/column/.test(span.className)) {
+			span.className = 'column' + (className || '');
 		}
 	}
 }
@@ -439,12 +425,14 @@ function columnMouse(className) {
 * @return boolean false
 */
 function selectSearch(name) {
-	var el = qs('#fieldset-search');
+	let el = qs('#fieldset-search');
 	el.className = '';
-	var divs = qsa('div', el);
-	for (var i=0; i < divs.length; i++) {
-		var div = divs[i];
-		if (isTag(div.firstChild, 'select') && selectValue(div.firstChild) == name) {
+	const divs = qsa('div', el);
+	let i, div;
+	for (i=0; i < divs.length; i++) {
+		div = divs[i];
+		el = qs('[name$="[col]"]', div);
+		if (el && selectValue(el) == name) {
 			break;
 		}
 	}
@@ -465,14 +453,6 @@ function isCtrl(event) {
 	return (event.ctrlKey || event.metaKey) && !event.altKey; // shiftKey allowed
 }
 
-/** Return event target
-* @param Event
-* @return HTMLElement
-*/
-function getTarget(event) {
-	return event.target || event.srcElement;
-}
-
 
 
 /** Send form by Ctrl+Enter on <select> and <textarea>
@@ -482,18 +462,16 @@ function getTarget(event) {
 */
 function bodyKeydown(event, button) {
 	eventStop(event);
-	var target = getTarget(event);
+	let target = event.target;
 	if (target.jushTextarea) {
 		target = target.jushTextarea;
 	}
 	if (isCtrl(event) && (event.keyCode == 13 || event.keyCode == 10) && isTag(target, 'select|textarea|input')) { // 13|10 - Enter
 		target.blur();
-		if (button) {
+		if (target.form[button]) {
 			target.form[button].click();
 		} else {
-			if (target.form.onsubmit) {
-				target.form.onsubmit();
-			}
+			target.form.dispatchEvent(new Event('submit', {bubbles: true}));
 			target.form.submit();
 		}
 		target.focus();
@@ -506,10 +484,10 @@ function bodyKeydown(event, button) {
 * @param MouseEvent
 */
 function bodyClick(event) {
-	var target = getTarget(event);
+	const target = event.target;
 	if ((isCtrl(event) || event.shiftKey) && target.type == 'submit' && isTag(target, 'input')) {
 		target.form.target = '_blank';
-		setTimeout(function () {
+		setTimeout(() => {
 			// if (isCtrl(event)) { focus(); } doesn't work
 			target.form.target = '';
 		}, 0);
@@ -518,15 +496,15 @@ function bodyClick(event) {
 
 
 
-/** Change focus by Ctrl+Up or Ctrl+Down
+/** Change focus by Ctrl+Shift+Up or Ctrl+Shift+Down
 * @param KeyboardEvent
 * @return boolean
 */
 function editingKeydown(event) {
 	if ((event.keyCode == 40 || event.keyCode == 38) && isCtrl(event)) { // 40 - Down, 38 - Up
-		var target = getTarget(event);
-		var sibling = (event.keyCode == 40 ? 'nextSibling' : 'previousSibling');
-		var el = target.parentNode.parentNode[sibling];
+		const target = event.target;
+		const sibling = (event.keyCode == 40 ? 'nextSibling' : 'previousSibling');
+		let el = target.parentNode.parentNode[sibling];
 		if (el && (isTag(el, 'tr') || (el = el[sibling])) && isTag(el, 'tr') && (el = el.childNodes[nodePosition(target.parentNode)]) && (el = el.childNodes[nodePosition(target)])) {
 			el.focus();
 		}
@@ -542,7 +520,7 @@ function editingKeydown(event) {
 * @this HTMLSelectElement
 */
 function functionChange() {
-	var input = this.form[this.name.replace(/^function/, 'fields')];
+	const input = this.form[this.name.replace(/^function/, 'fields')];
 	if (input) { // undefined with the set data type
 		if (selectValue(this)) {
 			if (input.origType === undefined) {
@@ -567,7 +545,7 @@ function functionChange() {
 * @this HTMLTableCellElement
 */
 function skipOriginal(first) {
-	var fnSelect = this.previousSibling.firstChild;
+	const fnSelect = qs('select', this.previousSibling);
 	if (fnSelect.selectedIndex < first) {
 		fnSelect.selectedIndex = first;
 	}
@@ -577,14 +555,13 @@ function skipOriginal(first) {
 * @this HTMLInputElement
 */
 function fieldChange() {
-	var row = cloneNode(parentTag(this, 'tr'));
-	var inputs = qsa('input', row);
-	for (var i = 0; i < inputs.length; i++) {
-		inputs[i].value = '';
+	const row = cloneNode(parentTag(this, 'tr'));
+	for (const input of qsa('input', row)) {
+		input.value = '';
 	}
 	// keep value in <select> (function)
 	parentTag(this, 'table').appendChild(row);
-	this.oninput = function () { };
+	this.oninput = () => { };
 }
 
 
@@ -598,27 +575,25 @@ function fieldChange() {
 * @uses offlineMessage
 */
 function ajax(url, callback, data, message) {
-	var request = (window.XMLHttpRequest ? new XMLHttpRequest() : (window.ActiveXObject ? new ActiveXObject('Microsoft.XMLHTTP') : false));
+	const request = new XMLHttpRequest();
 	if (request) {
-		var ajaxStatus = qs('#ajaxstatus');
+		const ajaxStatus = qs('#ajaxstatus');
 		if (message) {
 			ajaxStatus.innerHTML = '<div class="message">' + message + '</div>';
-			ajaxStatus.className = ajaxStatus.className.replace(/ hidden/g, '');
-		} else {
-			ajaxStatus.className += ' hidden';
 		}
+		alterClass(ajaxStatus, 'hidden', !message);
 		request.open((data ? 'POST' : 'GET'), url);
 		if (data) {
 			request.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
 		}
 		request.setRequestHeader('X-Requested-With', 'XMLHttpRequest');
-		request.onreadystatechange = function () {
+		request.onreadystatechange = () => {
 			if (request.readyState == 4) {
 				if (/^2/.test(request.status)) {
 					callback(request);
-				} else {
+				} else if (message !== null) {
 					ajaxStatus.innerHTML = (request.status ? request.responseText : '<div class="error">' + offlineMessage + '</div>');
-					ajaxStatus.className = ajaxStatus.className.replace(/ hidden/g, '');
+					alterClass(ajaxStatus, 'hidden');
 				}
 			}
 		};
@@ -632,14 +607,17 @@ function ajax(url, callback, data, message) {
 * @return boolean false for success
 */
 function ajaxSetHtml(url) {
-	return !ajax(url, function (request) {
-		var data = window.JSON ? JSON.parse(request.responseText) : eval('(' + request.responseText + ')');
-		for (var key in data) {
+	return !ajax(url, request => {
+		const data = JSON.parse(request.responseText);
+		for (const key in data) {
 			setHtml(key, data[key]);
 		}
 	});
 }
 
+let editChanged; // used by plugins
+let adminerHighlighter = els => {}; // overwritten by syntax highlighters
+
 /** Save form contents through AJAX
 * @param HTMLFormElement
 * @param string
@@ -647,10 +625,8 @@ function ajaxSetHtml(url) {
 * @return boolean
 */
 function ajaxForm(form, message, button) {
-	var data = [];
-	var els = form.elements;
-	for (var i = 0; i < els.length; i++) {
-		var el = els[i];
+	let data = [];
+	for (const el of form.elements) {
 		if (el.name && !el.disabled) {
 			if (/^file$/i.test(el.type) && el.value) {
 				return false;
@@ -661,17 +637,20 @@ function ajaxForm(form, message, button) {
 		}
 	}
 	data = data.join('&');
-	
-	var url = form.action;
+
+	let url = form.action;
 	if (!/post/i.test(form.method)) {
 		url = url.replace(/\?.*/, '') + '?' + data;
 		data = '';
 	}
-	return ajax(url, function (request) {
+	return ajax(url, request => {
+		const ajaxstatus = qs('#ajaxstatus');
 		setHtml('ajaxstatus', request.responseText);
-		if (window.jush) {
-			jush.highlight_tag(qsa('code', qs('#ajaxstatus')), 0);
+		if (qs('.message', ajaxstatus)) { // success
+			editChanged = null;
 		}
+		adminerHighlighter(qsa('code', ajaxstatus));
+		messagesPrint(ajaxstatus);
 	}, data, message);
 }
 
@@ -685,8 +664,8 @@ function ajaxForm(form, message, button) {
 * @this HTMLElement
 */
 function selectClick(event, text, warning) {
-	var td = this;
-	var target = getTarget(event);
+	const td = this;
+	const target = event.target;
 	if (!isCtrl(event) || isTag(td.firstChild, 'input|textarea') || isTag(target, 'a')) {
 		return;
 	}
@@ -694,24 +673,25 @@ function selectClick(event, text, warning) {
 		alert(warning);
 		return true;
 	}
-	var original = td.innerHTML;
+	const original = td.innerHTML;
 	text = text || /\n/.test(original);
-	var input = document.createElement(text ? 'textarea' : 'input');
-	input.onkeydown = function (event) {
-		if (!event) {
-			event = window.event;
-		}
+	const input = document.createElement(text ? 'textarea' : 'input');
+	input.onkeydown = event => {
 		if (event.keyCode == 27 && !event.shiftKey && !event.altKey && !isCtrl(event)) { // 27 - Esc
 			inputBlur.apply(input);
 			td.innerHTML = original;
 		}
 	};
-	var pos = event.rangeOffset;
-	var value = (td.firstChild && td.firstChild.alt) || td.textContent || td.innerText;
-	input.style.width = Math.max(td.clientWidth - 14, 20) + 'px'; // 14 = 2 * (td.border + td.padding + input.border)
+
+	const pos = getSelection().anchorOffset;
+	let value = (td.firstChild && td.firstChild.alt) || td.textContent;
+	const tdStyle = window.getComputedStyle(td, null);
+
+	input.style.width = Math.max(td.clientWidth - parseFloat(tdStyle.paddingLeft) - parseFloat(tdStyle.paddingRight), (text ? 200 : 20)) + 'px';
+
 	if (text) {
-		var rows = 1;
-		value.replace(/\n/g, function () {
+		let rows = 1;
+		value.replace(/\n/g, () => {
 			rows++;
 		});
 		input.rows = rows;
@@ -719,20 +699,12 @@ function selectClick(event, text, warning) {
 	if (qsa('i', td).length) { // <i> - NULL
 		value = '';
 	}
-	if (document.selection) {
-		var range = document.selection.createRange();
-		range.moveToPoint(event.clientX, event.clientY);
-		var range2 = range.duplicate();
-		range2.moveToElementText(td);
-		range2.setEndPoint('EndToEnd', range);
-		pos = range2.text.length;
-	}
 	td.innerHTML = '';
 	td.appendChild(input);
 	setupSubmitHighlight(td);
 	input.focus();
 	if (text == 2) { // long text
-		return ajax(location.href + '&' + encodeURIComponent(td.id) + '=', function (request) {
+		return ajax(location.href + '&' + encodeURIComponent(td.id) + '=', request => {
 			if (request.responseText) {
 				input.value = request.responseText;
 				input.name = td.id;
@@ -743,11 +715,6 @@ function selectClick(event, text, warning) {
 	input.name = td.id;
 	input.selectionStart = pos;
 	input.selectionEnd = pos;
-	if (document.selection) {
-		var range = document.selection.createRange();
-		range.moveEnd('character', -input.value.length + pos);
-		range.select();
-	}
 	return true;
 }
 
@@ -760,22 +727,20 @@ function selectClick(event, text, warning) {
 * @this HTMLLinkElement
 */
 function selectLoadMore(limit, loading) {
-	var a = this;
-	var title = a.innerHTML;
-	var href = a.href;
+	const a = this;
+	const title = a.innerHTML;
+	const href = a.href;
 	a.innerHTML = loading;
 	if (href) {
 		a.removeAttribute('href');
-		return !ajax(href, function (request) {
-			var tbody = document.createElement('tbody');
+		return !ajax(href, request => {
+			const tbody = document.createElement('tbody');
 			tbody.innerHTML = request.responseText;
 			qs('#table').appendChild(tbody);
 			if (tbody.children.length < limit) {
 				a.parentNode.removeChild(a);
 			} else {
-				a.href = href.replace(/\d+$/, function (page) {
-					return +page + 1;
-				});
+				a.href = href.replace(/\d+$/, page => +page + 1);
 				a.innerHTML = title;
 			}
 		});
@@ -788,11 +753,7 @@ function selectLoadMore(limit, loading) {
 * @param Event
 */
 function eventStop(event) {
-	if (event.stopPropagation) {
-		event.stopPropagation();
-	} else {
-		event.cancelBubble = true;
-	}
+	event.stopPropagation();
 }
 
 
@@ -801,11 +762,8 @@ function eventStop(event) {
 * @param HTMLElement
 */
 function setupSubmitHighlight(parent) {
-	for (var key in { input: 1, select: 1, textarea: 1 }) {
-		var inputs = qsa(key, parent);
-		for (var i = 0; i < inputs.length; i++) {
-			setupSubmitHighlightInput(inputs[i])
-		}
+	for (const input of qsa('input, select, textarea', parent)) {
+		setupSubmitHighlightInput(input);
 	}
 }
 
@@ -813,7 +771,7 @@ function setupSubmitHighlight(parent) {
 * @param HTMLElement
 */
 function setupSubmitHighlightInput(input) {
-	if (!/submit|image|file/.test(input.type)) {
+	if (!/submit|button|image|file/.test(input.type)) {
 		addEvent(input, 'focus', inputFocus);
 		addEvent(input, 'blur', inputBlur);
 	}
@@ -823,20 +781,14 @@ function setupSubmitHighlightInput(input) {
 * @this HTMLInputElement
 */
 function inputFocus() {
-	var submit = findDefaultSubmit(this);
-	if (submit) {
-		alterClass(submit, 'default', true);
-	}
+	alterClass(findDefaultSubmit(this), 'default', true);
 }
 
 /** Unhighlight default submit button
 * @this HTMLInputElement
 */
 function inputBlur() {
-	var submit = findDefaultSubmit(this);
-	if (submit) {
-		alterClass(submit, 'default');
-	}
+	alterClass(findDefaultSubmit(this), 'default');
 }
 
 /** Find submit button used by Enter
@@ -850,9 +802,7 @@ function findDefaultSubmit(el) {
 	if (!el.form) {
 		return null;
 	}
-	var inputs = qsa('input', el.form);
-	for (var i = 0; i < inputs.length; i++) {
-		var input = inputs[i];
+	for (const input of qsa('input', el.form)) {
 		if (input.type == 'submit' && !input.style.zIndex) {
 			return input;
 		}
@@ -867,20 +817,7 @@ function findDefaultSubmit(el) {
 * @param function
 */
 function addEvent(el, action, handler) {
-	if (el.addEventListener) {
-		el.addEventListener(action, handler, false);
-	} else {
-		el.attachEvent('on' + action, handler);
-	}
-}
-
-/** Defer focusing element
-* @param HTMLElement
-*/
-function focus(el) {
-	setTimeout(function () { // this has to be an anonymous function because Firefox passes some arguments to setTimeout callback
-		el.focus();
-	}, 0);
+	el.addEventListener(action, handler, false);
 }
 
 /** Clone node and setup submit highlighting
@@ -888,13 +825,13 @@ function focus(el) {
 * @return HTMLElement
 */
 function cloneNode(el) {
-	var el2 = el.cloneNode(true);
-	var selector = 'input, select';
-	var origEls = qsa(selector, el);
-	var cloneEls = qsa(selector, el2);
-	for (var i=0; i < origEls.length; i++) {
-		var origEl = origEls[i];
-		for (var key in origEl) {
+	const el2 = el.cloneNode(true);
+	const selector = 'input, select';
+	const origEls = qsa(selector, el);
+	const cloneEls = qsa(selector, el2);
+	for (let i=0; i < origEls.length; i++) {
+		const origEl = origEls[i];
+		for (const key in origEl) {
 			if (/^on/.test(key) && origEl[key]) {
 				cloneEls[i][key] = origEl[key];
 			}
@@ -904,8 +841,14 @@ function cloneNode(el) {
 	return el2;
 }
 
-oninput = function (event) {
-	var target = event.target;
-	var maxLength = target.getAttribute('data-maxlength');
+oninput = event => {
+	const target = event.target;
+	const maxLength = target.getAttribute('data-maxlength');
 	alterClass(target, 'maxlength', target.value && maxLength != null && target.value.length > maxLength); // maxLength could be 0
 };
+
+addEvent(document, 'click', event => {
+	if (!qs('#foot').contains(event.target)) {
+		alterClass(qs('#foot'), 'foot', true);
+	}
+});
